==> scoop_0.1-3.tar.gz: scoop/R/scoop.R <==
scoop <- function(x, ...) {
  UseMethod("scoop")
}

scoop.formula <- function(formula,
                          data,
                          family       = "gaussian",
                          penalty      = "coop", 
                          intercept    = TRUE,
                          normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                          lambda       = NULL,
                          n.lambda     = 100,
                          lambda.min   = 1e-3,
                          weights      = NULL,
                          verbose      = FALSE,
                          eps          = c(1e-5,1e-4),
                          max.iter     = NULL,
                          max.features = NULL,
                          optim.method = ifelse(family=="gaussian","fista","bfgs"),
                          ...) {

  mf <- match.call(expand.dots = FALSE)
  m  <- match(c("formula","data"), names(mf), 0L)
  mf <- mf[c(1L,m)]
  mf$drop.unused.levels <- TRUE
  mf[[1L]] <- as.name("model.frame")
  mf <- eval(mf, parent.frame())
  x  <- model.matrix(attr(mf,"terms"), data = mf, contrasts = NULL)
  y  <- model.response(mf)

  group <- attr(x, "assign")
  ## treating intercept
  if (0 %in% group) {
    group <- group[-1]
    intercept <- TRUE
    x <- x[,-1]
  } else {
    intercept <- FALSE
  }
  if (is.null(max.iter)) {
    max.iter <- 2*ncol(x)
  }
  if (is.null(max.features)) {
    max.features <- length(table(group))
  }
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = penalty, 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method   = optim.method)
  out$call  <- match.call()
  out$terms <- attr(mf,"terms")
  
  return(structure(out, class="scoop"))
}

lasso <- function(x,
                  y,
                  family       = "gaussian",
                  intercept    = TRUE,
                  normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                  weights      = NULL,
                  lambda       = NULL,
                  n.lambda     = 100,
                  lambda.min   = 1e-3,
                  verbose      = FALSE,
                  eps          = c(1e-5,1e-4),
                  max.iter     = 2*ncol(x),
                  max.features = ncol(x),
                  optim.method = ifelse(family=="gaussian","fista","bfgs"),
                  ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       family       = family,
                       penalty      = "lasso", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method = optim.method)
}

group.lasso <- function(x,
                        y,
                        group,
                        family       = "gaussian",
                        intercept    = TRUE,
                        normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                        weights      = NULL,
                        lambda       = NULL,
                        n.lambda     = 100,
                        lambda.min   = 1e-3,
                        verbose      = FALSE,
                        eps          = c(1e-5,1e-4),
                        max.iter     = 2*ncol(x),
                        max.features = length(table(group)),
                        optim.method = ifelse(family=="gaussian","fista","bfgs"),
                        ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = "group", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method = optim.method)
}

coop.lasso <- function(x,y,
                       group,
                       family       = "gaussian",
                       intercept    = TRUE,
                       normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                       weights      = NULL,
                       lambda       = NULL,
                       n.lambda     = 100,
                       lambda.min   = 1e-3,
                       verbose      = FALSE,
                       eps          = c(1e-5,1e-4),
                       max.iter     = 2*ncol(x),
                       max.features = length(table(group)),
                       optim.method = ifelse(family=="gaussian","fista","bfgs"),
                       ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = "coop", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method   = optim.method)
  
}

scoop.default <- function(x,
                          y,
                          group,
                          family       = "gaussian",
                          penalty      = "coop", 
                          intercept    = TRUE,
                          normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                          weights      = NULL,
                          lambda       = NULL,
                          n.lambda     = 100,
                          lambda.min   = 1e-3,
                          verbose      = FALSE,
                          eps          = c(1e-5,1e-4),
                          max.iter     = 2*ncol(x),
                          max.features = ifelse(missing(group),ncol(x),
                            length(table(group))),
                          optim.method = ifelse(family=="gaussian","fista","bfgs"),
                          ...) {
  
  ## Save x and y, put some colnames
  if (is.null(colnames(x))) {colnames(x) <- 1:ncol(x)}
  x.init <- as.matrix(x)
  y.init <- y

  ## Default os Lasso when no group structrue is provived
  if (missing(group)) {
    group <- 1:ncol(x)
    if (penalty != "lasso") {
      warning("The group vector is null: enforcing penalty to 'lasso'.")
      penalty <- "lasso"
    }
  }
  
  ## model's choice and environment setting
  model <- paste(family,penalty,sep="")
  method <- switch(model,
                   "gaussianlasso" = lm.las ,
                   "gaussiangroup" = lm.grp ,
                   "gaussiancoop"  = lm.coo ,
                   "binomiallasso" = lrm.las,
                   "binomialgroup" = lrm.grp,
                   "binomialcoop"  = lrm.coo, NULL)
  optim.func  <- switch(paste(optim.method,model,sep=""),
                        "bfgsgaussianlasso"  = bfgs.lm.las ,
                        "bfgsgaussiangroup"  = bfgs.lm.grp ,
                        "bfgsgaussiancoop"   = bfgs.lm.coo ,
                        "bfgsbinomiallasso"  = bfgs.lrm.las,
                        "bfgsbinomialgroup"  = bfgs.lrm.grp,
                        "bfgsbinomialcoop"   = bfgs.lrm.coo,
                        "istagaussianlasso"  = ista.lm.las ,
                        "istagaussiangroup"  = ista.lm.grp ,
                        "istagaussiancoop"   = ista.lm.coo ,
                        "istabinomiallasso"  = NULL,
                        "istabinomialgroup"  = NULL,
                        "istabinomialcoop"   = NULL,
                        "fistagaussianlasso" = fista.lm.las,
                        "fistagaussiangroup" = fista.lm.grp,
                        "fistagaussiancoop"  = fista.lm.coo,
                        "fistabinomiallasso" = NULL,
                        "fistabinomialgroup" = NULL,
                        "fistabinomialcoop"  = NULL, NULL)
  if (is.null(method)  | is.null(optim.func)) {
    stop("wrong type of model specified.")
  }

  if (is.null(optim.func)) {
    stop("Sorry, not yet available... working on it.")
  }
  
  ## ======================================================
  ## INTERCEPT - TREATMENT
  ## Gaussian case
  if (intercept & family == "gaussian") {
    y.bar <- mean(y)
    y <- y-y.bar
    x.bar <- colMeans(x)
    x     <- scale(x,x.bar,FALSE)
    mu.start <- NULL
  }
  ## Binomial family
  if (family == "binomial") {
    mu   <- c()
    if (intercept) {
      mu.start <- 0
      x.bar <- colMeans(x)
      x <- scale(x,x.bar,FALSE)
    } else {
      mu.start <- NULL
    }
  }

  ## ======================================================
  ## NORMALIZATION, REORDERING
  ## normalizing the data
  if (normalize) {
    norm <- sqrt(drop(colSums(x^2)))
    x <- scale(x, FALSE, norm)
  } else {
    norm <- rep(1,ncol(x))
  } 
  ## sorting the groups and the columns of the design matrix
  o       <- order(group,decreasing=FALSE)
  x       <- x[,o]
  group   <- group[o]
  tpk     <- tabulate(factor(group))

  ## ======================================================
  ## GENERATE A GRID OF LAMBDA IF NONE HAS BEEN PROVIDED
  if (is.null(lambda)) {
    if (family == "binomial" & intercept) {
      xTy <- t(x) %*% (y-mean(y))
    } else {
      xTy <- t(x) %*% y
    }
    if (penalty == "coop") {
      n2p  <- group.norm(pmax.int(0,xTy),tpk)
      n2m  <- group.norm(pmin.int(0,xTy),tpk)
      nmax <- pmax.int(n2p,n2m)
    } else {
      nmax <- group.norm(xTy,tpk)
    }
    lambda.max <- max(nmax / sqrt(tpk))
    lambda <- 10^(seq(from=log10(lambda.max),to=log10(lambda.min),
                      length=n.lambda))
  }
  ## if the vector of weight is null, apply a scheme which penalize
  ## according to the group size
  if (is.null(weights)) {
    weights <- rep.int(sqrt(tpk),tpk)
  }
  
  ## ========================================================
  ## LOOP OVER LAMBDA
  if (length(eps) > 1){
    eps.outer <- eps[1]
    eps.inner <- eps[2]
  } else {
    eps.outer <- eps
    eps.inner <- eps
  }  
  iter.optim <- c()
  iter  <- c()
  dual.gap <- c()
  beta0 <- rep(0,ncol(x))
  beta.start <- beta0
  last.beta  <- rep(-1,ncol(x))
  beta <- c()
  real.lambda <- c()
  ## quantities which won't move along the regularization path
  ## differs weither the family is gaussian or binomial
  fx <- switch(family, "gaussian" = crossprod(x,x)   , "binomial" = x)
  fy <- switch(family, "gaussian" = c(crossprod(y,x)), "binomial" = y)
  for (l in lambda) {
    if (verbose) cat("\nlambda =",l)
    ## CALL TO THE FUNCTION THAT DOES THE WORK
    out <- method(fx,fy,group,l*weights,beta.start,mu.start,
                  eps.outer,eps.inner,max.iter,optim.func)
    iter <- c(iter,length(out$iter.optim))
    iter.optim <- c(iter.optim,out$iter.optim)
    dual.gap <- c(dual.gap,out$gap)
    if (out$status == "converged") {
      beta.start <- out$beta
      if (family == "binomial") {mu.start <- out$mu}
    } else {
      beta.start <- beta0
      if (family == "binomial") {
        if (intercept) {
          mu.start <- 0
        } else {
          mu.start <- NULL
        }        
      }
    }

    ## When the path of solutions has stabilized, try to earn some time
    ## by avoiding irreleavant computations
    if (sqrt(sum((last.beta-out$beta)^2)) < eps.outer & !all(last.beta == 0)) {
      missing <- n.lambda - length(real.lambda)
      beta <- rbind(beta,t(matrix(rep(out$beta,missing),ncol=missing)))
      if (family == "binomial") {
        mu <- c(mu,rep(out$mu,missing))
      }
      real.lambda <- lambda
      break
    }

    ## Move to the next lambda
    beta  <- rbind(beta,out$beta)
    real.lambda <- c(real.lambda,l)
    last.beta <- out$beta
    if (family == "binomial") {mu <- c(mu,out$mu)}

    ## When the algorithm does not converge, send a message and start over from
    ## zero for the next iterate
    if (verbose & out$status != "converged") {
      cat("\nscoop:",out$status, "with max.min||subgradient|| =", out$gap)
      beta.start <- rep(0,ncol(x))
      if (family == "binomial" & intercept) {mu.start <- 0}
    }

    ## If the number of features required by the user has been reached, then stop
    if(sum(group.norm.unordered(out$beta,group) !=0) > max.features ) {
      break
    }
  } ## END OF THE LOOP OVER LAMBDA
  lambda <- real.lambda
  ## ======================================================
  ## RESCALING, REORDERING
  ## unsorting beta
  group <- group[order(o)]
  if (is.null(dim(beta))) {
    beta  <- beta[order(o)]
  } else {
    beta  <- beta[, order(o)]
  }
  x     <- x[,order(o)]
  ## re-scaling beta
  if (normalize) {
    if (length(lambda) > 1) {
      beta <- scale(as.matrix(beta), FALSE, norm)
    } else {
      beta <- beta /  norm
    }
  }
  beta <- matrix(beta,ncol=ncol(x), nrow=length(lambda))

  ## ======================================================
  ## INTERCEPT TREATMENT
  if (intercept) {
    if (family == "gaussian") {
      mu   <- y.bar -  beta %*% x.bar
    } else {
      mu <- mu - apply(t(beta) * x.bar, 2, sum)
    }
    beta <- cbind(mu,beta)    
    if (length(lambda) == 1) {
      names(beta) <- c("intercept",colnames(x))
    } else {
      colnames(beta) <- c("intercept",colnames(x))
      rownames(beta) <- round(lambda,2)
    }
  } else {    
    if (length(lambda) == 1) {
      names(beta) <- colnames(x)
    } else {
      colnames(beta) <- colnames(x)
      rownames(beta) <- round(lambda,2)
    }
  }

  ## The regular fit is used for df approximation in the linear regression case
  reg.fit <- suppressWarnings(reg.fitting(x.init, y.init, intercept, family))
  
  out <- list(coefficients  = beta,
              lambda        = lambda,
              group         = group,
              family        = family,
              penalty       = penalty,
              reg.fit       = reg.fit,
              x             = x.init,
              y             = y.init,
	      it	    = iter,
	      it.optim      = iter.optim,
              dual.gap      = dual.gap,
              call          = match.call())
  
  return(structure(out, class="scoop"))
}

==> scoop_0.1-4.tar.gz: scoop/R/scoop.R <==
scoop <- function(x, ...) {
  UseMethod("scoop")
}

scoop.formula <- function(formula,
                          data,
                          family       = "gaussian",
                          penalty      = "coop", 
                          intercept    = TRUE,
                          normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                          lambda       = NULL,
                          n.lambda     = 100,
                          lambda.min   = 1e-2,
                          weights      = NULL,
                          verbose      = FALSE,
                          eps          = c(1e-5,1e-4),
                          max.iter     = NULL,
                          max.features = NULL,
                          optim.method = ifelse(family=="gaussian","fista","bfgs"),
                          ...) {

  mf <- match.call(expand.dots = FALSE)
  m  <- match(c("formula","data"), names(mf), 0L)
  mf <- mf[c(1L,m)]
  mf$drop.unused.levels <- TRUE
  mf[[1L]] <- as.name("model.frame")
  mf <- eval(mf, parent.frame())
  x  <- model.matrix(attr(mf,"terms"), data = mf, contrasts = NULL)
  y  <- model.response(mf)

  group <- attr(x, "assign")
  ## treating intercept
  if (0 %in% group) {
    group <- group[-1]
    intercept <- TRUE
    x <- x[,-1]
  } else {
    intercept <- FALSE
  }
  if (is.null(max.iter)) {
    max.iter <- 2*ncol(x)
  }
  if (is.null(max.features)) {
    max.features <- length(table(group))
  }
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = penalty, 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method   = optim.method)
  out$call  <- match.call()
  out$terms <- attr(mf,"terms")
  
  return(structure(out, class="scoop"))
}

lasso <- function(x,
                  y,
                  family       = "gaussian",
                  intercept    = TRUE,
                  normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                  weights      = NULL,
                  lambda       = NULL,
                  n.lambda     = 100,
                  lambda.min   = 1e-2,
                  verbose      = FALSE,
                  eps          = c(1e-5,1e-4),
                  max.iter     = 2*ncol(x),
                  max.features = ncol(x),
                  optim.method = ifelse(family=="gaussian","fista","bfgs"),
                  ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       family       = family,
                       penalty      = "lasso", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method = optim.method)
}

group.lasso <- function(x,
                        y,
                        group,
                        family       = "gaussian",
                        intercept    = TRUE,
                        normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                        weights      = NULL,
                        lambda       = NULL,
                        n.lambda     = 100,
                        lambda.min   = 1e-2,
                        verbose      = FALSE,
                        eps          = c(1e-5,1e-4),
                        max.iter     = 2*ncol(x),
                        max.features = length(table(group)),
                        optim.method = ifelse(family=="gaussian","fista","bfgs"),
                        ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = "group", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method = optim.method)
}

coop.lasso <- function(x,y,
                       group,
                       family       = "gaussian",
                       intercept    = TRUE,
                       normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                       weights      = NULL,
                       lambda       = NULL,
                       n.lambda     = 100,
                       lambda.min   = 1e-2,
                       verbose      = FALSE,
                       eps          = c(1e-5,1e-4),
                       max.iter     = 2*ncol(x),
                       max.features = length(table(group)),
                       optim.method = ifelse(family=="gaussian","fista","bfgs"),
                       ...) {
  
  out <- scoop.default(x = x,
                       y = y,
                       group        = group,
                       family       = family,
                       penalty      = "coop", 
                       intercept    = intercept,
                       normalize    = normalize,
                       weights      = weights,
                       lambda       = lambda,
                       n.lambda     = n.lambda,
                       lambda.min   = lambda.min,
                       verbose      = verbose,
                       eps          = eps,
                       max.iter     = max.iter,
                       max.features = max.features,
                       optim.method   = optim.method)
  
}

scoop.default <- function(x,
                          y,
                          group,
                          family       = "gaussian",
                          penalty      = "coop", 
                          intercept    = TRUE,
                          normalize    = ifelse(family=="gaussian",TRUE,FALSE),
                          weights      = NULL,
                          lambda       = NULL,
                          n.lambda     = 100,
                          lambda.min   = 1e-2,
                          verbose      = FALSE,
                          eps          = c(1e-5,1e-4),
                          max.iter     = 2*ncol(x),
                          max.features = ifelse(missing(group),ncol(x),
                            length(table(group))),
                          optim.method = ifelse(family=="gaussian","fista","bfgs"),
                          ...) {
  
  ## Save x and y, put some colnames
  if (is.null(colnames(x))) {colnames(x) <- 1:ncol(x)}
  x.init <- as.matrix(x)
  y.init <- y

  ## Default os Lasso when no group structrue is provived
  if (missing(group)) {
    group <- 1:ncol(x)
    if (penalty != "lasso") {
      warning("The group vector is null: enforcing penalty to 'lasso'.")
      penalty <- "lasso"
    }
  }
  
  ## model's choice and environment setting
  model <- paste(family,penalty,sep="")
  method <- switch(model,
                   "gaussianlasso" = lm.las ,
                   "gaussiangroup" = lm.grp ,
                   "gaussiancoop"  = lm.coo ,
                   "binomiallasso" = lrm.las,
                   "binomialgroup" = lrm.grp,
                   "binomialcoop"  = lrm.coo, NULL)
  optim.func  <- switch(paste(optim.method,model,sep=""),
                        "bfgsgaussianlasso"  = bfgs.lm.las ,
                        "bfgsgaussiangroup"  = bfgs.lm.grp ,
                        "bfgsgaussiancoop"   = bfgs.lm.coo ,
                        "bfgsbinomiallasso"  = bfgs.lrm.las,
                        "bfgsbinomialgroup"  = bfgs.lrm.grp,
                        "bfgsbinomialcoop"   = bfgs.lrm.coo,
                        "istagaussianlasso"  = ista.lm.las ,
                        "istagaussiangroup"  = ista.lm.grp ,
                        "istagaussiancoop"   = ista.lm.coo ,
                        "istabinomiallasso"  = ista.lrm.las,
                        "istabinomialgroup"  = ista.lrm.grp,
                        "istabinomialcoop"   = ista.lrm.coo,
                        "fistagaussianlasso" = fista.lm.las,
                        "fistagaussiangroup" = fista.lm.grp,
                        "fistagaussiancoop"  = fista.lm.coo,
                        "fistabinomiallasso" = fista.lrm.las,
                        "fistabinomialgroup" = fista.lrm.grp,
                        "fistabinomialcoop"  = fista.lrm.coo, NULL)
  if (is.null(method)  | is.null(optim.func)) {
    stop("wrong type of model specified.")
  }

  ## ======================================================
  ## INTERCEPT - TREATMENT
  ## Gaussian case
  if (intercept & family == "gaussian") {
    y.bar <- mean(y)
    y <- y-y.bar
    x.bar <- colMeans(x)
    x     <- scale(x,x.bar,FALSE)
    mu.start <- NULL
  }
  ## Binomial family
  if (family == "binomial") {
    if (!all(c("0","1") %in% levels(as.factor(y)))) {
      stop("y should only contain 0 and 1 in the binomial case")
    }
    mu   <- c()
    if (intercept) {
      mu.start <- log(sum(y==1)/sum(y==0))
      x.bar <- colMeans(x)
      x <- scale(x,x.bar,FALSE)
    } else {
      mu.start <- c()
    }
  }

  ## ======================================================
  ## NORMALIZATION, REORDERING
  ## normalizing the data
  if (normalize) {
    norm <- sqrt(drop(colSums(x^2)))
    x <- scale(x, FALSE, norm)
  } else {
    norm <- rep(1,ncol(x))
  } 
  ## sorting the groups and the columns of the design matrix
  o       <- order(group,decreasing=FALSE)
  x       <- x[,o]
  group   <- group[o]
  tpk     <- tabulate(factor(group))

  ## ======================================================
  ## GENERATE A GRID OF LAMBDA IF NONE HAS BEEN PROVIDED
  if (is.null(lambda)) {
    if (family == "binomial" & intercept) {
      xTy <- t(x) %*% (y-mean(y))
    } else {
      xTy <- t(x) %*% y
    }
    if (penalty == "coop") {
      n2p  <- group.norm(pmax.int(0,xTy),tpk)
      n2m  <- group.norm(pmin.int(0,xTy),tpk)
      nmax <- pmax.int(n2p,n2m)
    } else {
      nmax <- group.norm(xTy,tpk)
    }
    lambda.max <- max(nmax / sqrt(tpk))
    lambda <- 10^(seq(from=log10(lambda.max),to=log10(lambda.min),
                      length=n.lambda))
  }
  ## if the vector of weight is null, apply a scheme which penalize
  ## according to the group size
  if (is.null(weights)) {
    weights <- rep.int(sqrt(tpk),tpk)
  }
  
  ## ========================================================
  ## LOOP OVER LAMBDA
  if (length(eps) > 1){
    eps.outer <- eps[1]
    eps.inner <- eps[2]
  } else {
    eps.outer <- eps
    eps.inner <- eps
  }  
  iter.optim <- c()
  iter  <- c()
  dual.gap <- c()
  beta0 <- rep(0,ncol(x))
  beta.start <- beta0
  last.beta  <- rep(-1,ncol(x))
  beta <- c()
  real.lambda <- c()
  ## quantities which won't move along the regularization path
  ## differs weither the family is gaussian or binomial
  fx <- switch(family, "gaussian" = crossprod(x,x)   , "binomial" = x)
  fy <- switch(family, "gaussian" = c(crossprod(y,x)), "binomial" = y)
  for (l in lambda) {
    if (verbose) cat("\nlambda =",l)
    ## CALL TO THE FUNCTION THAT DOES THE WORK
    out <- method(fx,fy,group,l*weights,beta.start,mu.start,
                  eps.outer,eps.inner,max.iter,optim.func)
    iter <- c(iter,length(out$iter.optim))
    iter.optim <- c(iter.optim,out$iter.optim)
    dual.gap <- c(dual.gap,out$gap)
    beta.start <- out$beta
    if (family == "binomial") {
      mu.start <- out$mu
    }
    ## When the path of solutions has stabilized, try to earn some time
    ## by avoiding irreleavant computations
    if (sqrt(sum((last.beta-out$beta)^2)) < eps.outer & !all(last.beta == 0)) {
      missing <- n.lambda - length(real.lambda)
      beta <- rbind(beta,t(matrix(rep(out$beta,missing),ncol=missing)))
      if (family == "binomial") {
        mu <- c(mu,rep(out$mu,missing))
      }
      real.lambda <- lambda
      break
    }

    ## Move to the next lambda
    beta  <- rbind(beta,out$beta)
    real.lambda <- c(real.lambda,l)
    last.beta <- out$beta
    if (family == "binomial") {mu <- c(mu,out$mu)}
    
    ## When the algorithm does not converge, send a message and start over from
    ## zero for the next iterate
    if (verbose & out$status != "converged") {
      cat("\nscoop:",out$status, "with max.min||subgradient|| =", out$gap)
      beta.start <- rep(0,ncol(x))
      if (family == "binomial" & intercept) {mu.start <- 0}
    }
    
    ## If the number of features required by the user has been reached, then stop
    if(sum(group.norm.unordered(out$beta,group) !=0) > max.features ) {
      break
    }
  } ## END OF THE LOOP OVER LAMBDA
  lambda <- real.lambda
  names(iter.optim) <- NULL

  ## ======================================================
  ## RESCALING, REORDERING
  ## unsorting beta
  group <- group[order(o)]
  if (is.null(dim(beta))) {
    beta  <- beta[order(o)]
  } else {
    beta  <- beta[, order(o)]
  }
  x     <- x[,order(o)]
  ## re-scaling beta
  if (normalize) {
    if (length(lambda) > 1) {
      beta <- scale(as.matrix(beta), FALSE, norm)
    } else {
      beta <- beta /  norm
    }
  }
  beta <- matrix(beta,ncol=ncol(x), nrow=length(lambda))

  ## ======================================================
  ## INTERCEPT TREATMENT
  if (intercept) {
    if (family == "gaussian") {
      mu   <- y.bar -  beta %*% x.bar
    } else {
      mu <- mu - apply(t(beta) * x.bar, 2, sum)
    }
    beta <- cbind(mu,beta)    
    if (length(lambda) == 1) {
      names(beta) <- c("intercept",colnames(x))
    } else {
      colnames(beta) <- c("intercept",colnames(x))
      rownames(beta) <- round(lambda,2)
    }
  } else {    
    if (length(lambda) == 1) {
      names(beta) <- colnames(x)
    } else {
      colnames(beta) <- colnames(x)
      rownames(beta) <- round(lambda,2)
    }
  }

  ## The regular fit is used for df approximation in the linear regression case
  reg.fit <- suppressWarnings(reg.fitting(x.init, y.init, intercept, family))
  
  out <- list(coefficients  = beta,
              lambda        = lambda,
              group         = group,
              family        = family,
              penalty       = penalty,
              reg.fit       = reg.fit,
              x             = x.init,
              y             = y.init,
	      it	    = iter,
	      it.optim      = iter.optim,
              dual.gap      = dual.gap,
              call          = match.call())
  
  return(structure(out, class="scoop"))
}
